import { Component } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var Tree = /** @class */ (function (_super) {
    __extends(Tree, _super);
    function Tree() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            tree: []
        };
        _this.treeMap = {};
        _this.toggleNode = function (event) {
            event.stopPropagation();
            _this.extendOnToggle(event.currentTarget.id);
        };
        _this.createTreeExtender = function (customExtenderFn) {
            return function extender(data) {
                return data.reduce(function (result, node) {
                    return result.concat([
                        __assign({}, node, customExtenderFn(node), { children: node.children ? extender(node.children) : null })
                    ]);
                }, []);
            };
        };
        _this.setTreeState = function (treeState) {
            _this.setState({
                tree: treeState
            });
        };
        _this.initializeTree = function (activeId) {
            _this.treeMap = _this.createTreeMap(_this.props.config);
            var initialTreeState = _this.createTreeExtender(function (node) { return ({
                active: activeId && _this.treeMap[activeId].includes(node.id) ? true : false
            }); })(_this.props.config);
            _this.setTreeState(initialTreeState);
        };
        _this.extendOnToggle = function (nodeId) {
            var toggledTreeState = _this.createTreeExtender(function (node) { return ({
                active: nodeId === node.id ? !node.active : node.active
            }); })(_this.state.tree);
            _this.setTreeState(toggledTreeState);
        };
        _this.createTreeMap = function (data, parentNodePath) {
            return data.reduce(function (resultTreeMap, node) {
                var nodePath = parentNodePath
                    ? parentNodePath + ":" + node.id
                    : node.id;
                resultTreeMap[node.id] = nodePath;
                if (node.children) {
                    resultTreeMap = __assign({}, resultTreeMap, _this.createTreeMap(node.children, nodePath));
                }
                return resultTreeMap;
            }, {});
        };
        return _this;
    }
    Tree.prototype.componentDidMount = function () {
        this.initializeTree(this.props.activeId);
    };
    Tree.prototype.render = function () {
        var _this = this;
        return this.props.children({
            api: {
                toggleNode: this.toggleNode,
                getTree: function () { return _this.state.tree; }
            }
        });
    };
    return Tree;
}(Component));

export { Tree };
